<?php
/*
Plugin Name: WeLovePlugins - Plugin/Divi Downloader
Description: Descarga una lista de plugins básicos y elimina los plugins por defecto. Pueden añadirse o editarse los plugins a descargar en el archivo "plugins.txt" incluído en este plugin. Tambíen descarga Divi Theme y elimina los temas por defecto.
Version: 1.5
Requires at least: 2.5.1
Requires PHP: 5.2
Author: Christian Cabrero, Alberto Rodellar
Author URI: https://cabrero.ch
License: GPL v2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: cabrero-plugins
Domain Path: /languages
*/

    defined( 'ABSPATH' ) or die( 'No script kiddies please!' );
    


    function get_plugin_list_to_download() {

        $listado_plugins = plugin_dir_url( __FILE__ ).'plugins.txt';
        // You have to require following file in the front-end only. In the back-end; its already included

        if(isset($listado_plugins)){

            //return file_get_contents($listado_plugins);

            //Añadimos en un array por cada nueva linea del archivo
            return file($listado_plugins, FILE_IGNORE_NEW_LINES);

        } else {
            $default_content = "<br/>Por favor, comprueba que existe el archivo de lista de descarga.";
            return $default_content;
        }
 }



// El plugin define una función para descargar un plugin.
function descarga_plugin($plugin_slug) {

    // Obtenemos la URL del plugin.
    $plugin_url = plugins_url('/%s.zip', 'https://wordpress.org/plugins/sin-comentarios');

    // Descargamos el plugin.
    $fp = fopen($plugin_url, 'rb');
    $plugin_data = stream_get_contents($fp);
    fclose($fp);

    // Guardamos el plugin en la carpeta de plugins.
    $plugin_file = plugin_dir_path(__FILE__) . '/plugins/%s.zip';
    file_put_contents($plugin_file, $plugin_data);

    // Descomprimimos el plugin.
    $zip = new ZipArchive();
    $zip->open($plugin_file);
    $zip->extractTo(plugin_dir_path(__FILE__) . '/plugins/');
    $zip->close();

    // Eliminamos el archivo zip.
    unlink($plugin_file);

    // Activamos el plugin.
    wp_activate_plugin(basename($plugin_file, '.zip'));
}

// El plugin define una función para activar un plugin.
function activa_plugin($plugin_slug) {

    // Activamos el plugin.
    wp_activate_plugin($plugin_slug);
}

// El plugin define un menú de administración.
function plugin_menu() {

    // Añadimos un nuevo menú de administración.
    add_menu_page(
        'Descargar plugins',
        'Descargar plugins',
        'manage_options',
        'descargar-activar-plugins',
        'download_admin',
        'dashicons-download'
    );
}

// Esto llama a las dos funciones de instalar plugins y temas
function download_admin() {
    plugin_admin();
    theme_admin();
}

// El plugin define la página de administración.
function plugin_admin() {

    // Obtenemos los plugins disponibles.
    //$plugins = get_plugins();
    //print_r(get_plugin_list_to_download());

    // eliminamos los plugins por defecto hello dolly y akismet
    echo '<h2>Se han eliminado los siguientes plugins:</h2>';
    uninstall_default_plugins();


    $lista_plugins = get_plugin_list_to_download();

    echo '<h2>Se han descargado los siguientes plugins:</h2>';

    //Lista actualizada: https://wordpress.stackexchange.com/questions/25505/get-plugin-download-url-from-slug
    foreach ($lista_plugins as $plugin) {
        echo $plugin . '<br/>';

        // Descargamos el plugin.
        $fp = @fopen($plugin, 'rb');
        if ($fp === false) {
            echo '<b>Error al descargar el plugin:</b> ' . $plugin . '<br/><br/>';
            continue;
        }
        $plugin_data = stream_get_contents($fp);
        fclose($fp);

        // Guardamos el plugin en la carpeta de plugins.
        $plugin_file = plugin_dir_path(__FILE__) . 'temp-plugin.zip';
        file_put_contents($plugin_file, $plugin_data);
        // Descomprimimos el plugin.
        $zip = new ZipArchive();
        $zip->open($plugin_file);

        //Directorio dentro de este plugin
        //$zip->extractTo(plugin_dir_path(__FILE__) . '/plugins/');

        //Directorio de plugins general
        $plugins_root = WP_CONTENT_DIR . '/plugins';
        $zip->extractTo($plugins_root);

        $zip->close();
        // Delete the plugin file
        unlink($plugin_file);
    }
    echo '<br/><b>Ya puedes activarlos.</b><br/><br/>';
    echo '<a href="' . admin_url('plugins.php?plugin_status=inactive') . '"><button class="button button-primary">Activar plugins</button></a>';

    /*
    $dir = plugin_dir_path(__FILE__).'/plugins/';
    // Get array of all source files
    $files = scandir($dir);
    // Identify directories
    $source = $dir."/";
    $destination = WP_PLUGIN_DIR;
    // Cycle through all source files
    foreach ($files as $file) {
    if (in_array($file, array(".",".."))) continue;
    // If we copied this successfully, mark it for deletion
    if (copy($source.$file, $destination.$file)) {
        $delete[] = $source.$file;
    }
    }
    // Delete all successfully-copied files
    foreach ($delete as $file) {
    unlink($file);
    }*/


    // Mostramos una lista de plugins disponibles.
    /*foreach ($plugins as $plugin) {

        // Mostramos un enlace para descargar el plugin.
        echo '<a href="?action=descargar&slug=' . $plugin['slug'] . '">Descargar ' . $plugin['name'] . '</a>';

        // Mostramos un enlace para activar el plugin.
        echo '<a href="?action=activar&slug=' . $plugin['slug'] . '">Activar ' . $plugin['name'] . '</a>';
    }*/
}

// Registramos el menú de administración.
add_action('admin_menu', 'plugin_menu');

// Register the plugin with WordPress
function redirect_after_activation() {
    add_option('redirect_after_activation_option', true);
}
register_activation_hook(__FILE__, 'redirect_after_activation');

// Activation hook
function activation_redirect() {
    if (get_option('redirect_after_activation_option', false)) {
        delete_option('redirect_after_activation_option');
        exit(wp_redirect(admin_url('admin.php?page=descargar-activar-plugins')));
    }
}
add_action('admin_init', 'activation_redirect');

function uninstall_default_plugins()
{
    // se pueden añadir más plugins por defecto
    $default_plugins = array(
      'hello-dolly/hello.php',
      'akismet/akismet.php'
      );
      delete_plugins($default_plugins);
    foreach ($default_plugins as $plugin) {
        if (is_plugin_installed(explode('/', $plugin)[0])) {
            echo '<p>El plugin <strong>' . explode('/', $plugin)[0] . '</strong> no se ha eliminado.</p>';
        } else {
            echo '<p>El plugin <strong>' . explode('/', $plugin)[0] . '</strong> se ha eliminado correctamente.</p>';
        }
    }
}

// Función para verificar si el plugin está instalado
function is_plugin_installed($plugin)
{
    return is_dir(WP_PLUGIN_DIR . '/' . $plugin);
}


// Theme downloader:

// Función que instala y activa el tema personalizado
function theme_admin()
{
    $theme_url = 'https://welovewebs.com/descargas/Divi.zip'; // Enlace público de descarga a tu tema .zip
    $theme_name = 'Divi'; // Nombre de tu tema

    // Comprueba si el tema ya está instalado
    if (is_theme_installed($theme_name)) {
        echo '<p>El tema <strong>' . $theme_name . '</strong> ya está instalado y activado.</p>';
    } else {
        // Descarga el archivo .zip
        $downloaded_file = download_theme($theme_url);

        // Instala y activa el tema
        if ($downloaded_file) {
            $result = install_theme($downloaded_file);
            if ($result) {
                echo '<p>El tema <strong>' . $theme_name . '</strong> se ha instalado y activado correctamente.</p>';
            } else {
                echo '<p>Ocurrió un error al instalar el tema.</p>';
            }
        } else {
            echo '<p>Ocurrió un error al descargar el tema.</p>';
        }
    }
    uninstall_default_themes();
}

// Función para verificar si el tema está instalado
function is_theme_installed($theme_name)
{
    return is_dir(get_theme_root() . '/' . $theme_name);
}

// Función para descargar el tema desde el enlace .zip
function download_theme($theme_url)
{
    $upload_dir = wp_upload_dir();
    $download_file = $upload_dir['basedir'] . '/Divi.zip';

    $result = file_put_contents($download_file, fopen($theme_url, 'r'));

    return $download_file;
}

// Función para instalar y activar el tema desde el archivo .zip
function install_theme($theme_file)
{
    $wp_filesystem = WP_Filesystem();
    $theme_dir = get_theme_root();

    if ($wp_filesystem && file_exists($theme_file)) {
        $result = unzip_file($theme_file, $theme_dir);
        if (!$result) {
            return false;
        }
        $theme_name = basename($theme_file, '.zip');
        switch_theme($theme_name);
        unlink($theme_file);
        return true;
    }

    return false;
}

// Función para eliminar los temas por defecto
function uninstall_default_themes()
{
    // se pueden añadir más temas por defecto
    $default_themes = array(
    'twentytwentyfour',
    'twentytwentythree',
    'twentytwentytwo',
    'twentytwentyone',
    'twentytwenty'
    );
    foreach ($default_themes as $nombre_tema) {
      // Comprobar si el tema está instalado
        if (is_theme_installed($nombre_tema)) {
            if (delete_theme($nombre_tema)) {
                echo '<p>El tema <strong>' . $nombre_tema . '</strong> se ha eliminado correctamente.</p>';
            } else {
                echo '<p>Ocurrió un error al eliminar el tema <strong>' . $nombre_tema . '</strong>.</p>';
            }
        } else {
            echo '<p>El tema <strong>' . $nombre_tema . '</strong> no está instalado.</p>';
        }
    }
}
